@@ -19,7 +19,7 @@ from jinja2.optimizer import Optimizer
 from jinja2.exceptions import TemplateAssertionError
 from jinja2.utils import Markup, concat, escape
 from jinja2._compat import range_type, text_type, string_types, \
-     iteritems, NativeStringIO, imap
+     iteritems, NativeStringIO, imap, izip
 from jinja2.idtracking import Symbols, VAR_LOAD_PARAMETER, \
      VAR_LOAD_RESOLVE, VAR_LOAD_ALIAS, VAR_LOAD_UNDEFINED
 
@@ -796,7 +796,8 @@ class CodeGenerator(NodeVisitor):
         context = node.scoped and (
             'context.derived(%s)' % self.dump_local_context(frame)) or 'context'
 
-        if supports_yield_from and not self.environment.is_async:
+        if supports_yield_from and not self.environment.is_async and \
+           frame.buffer is None:
             self.writeline('yield from context.blocks[%r][0](%s)' % (
                            node.name, context), node)
         else:
@@ -1164,6 +1165,18 @@ class CodeGenerator(NodeVisitor):
         self.end_write(frame)
         self.leave_frame(filter_frame)
 
+    def visit_With(self, node, frame):
+        with_frame = frame.inner()
+        with_frame.symbols.analyze_node(node)
+        self.enter_frame(with_frame)
+        for idx, (target, expr) in enumerate(izip(node.targets, node.values)):
+            self.newline()
+            self.visit(target, with_frame)
+            self.write(' = ')
+            self.visit(expr, frame)
+        self.blockvisit(node.body, with_frame)
+        self.leave_frame(with_frame)
+
     def visit_ExprStmt(self, node, frame):
         self.newline(node)
         self.visit(node.node, frame)
